/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2003, 2004 Ralf Baechle
 * Copyright (C) 2004  Maciej W. Rozycki
 * Copyright (C) 2020 Loongson Technology Corporation Limited
 */
#ifndef __ASM_CPU_FEATURES_H
#define __ASM_CPU_FEATURES_H

#include <asm/cpu.h>
#include <asm/cpu-info.h>
#include <asm/isa-rev.h>
#include <cpu-feature-overrides.h>

#ifdef CONFIG_CPU_LOONGSON3
#include <loongson_cpu.h>
#endif

#define __ase(ase)			(cpu_data[0].ases & (ase))
#define __opt(opt)			(cpu_data[0].options & (opt))

/*
 * Check if LOONGARCH_ISA_REV is >= isa *and* an option or ASE is detected during
 * boot (typically by cpu_probe()).
 *
 * Note that these should only be used in cases where a kernel built for an
 * older ISA *cannot* run on a CPU which supports the feature in question. For
 * example this may be used for features introduced with LOONGARCHr6, since a kernel
 * built for an older ISA cannot run on a LOONGARCHr6 CPU. This should not be used
 * for LOONGARCHr2 features however, since a LOONGARCHr1 or earlier kernel might run on a
 * LOONGARCHr2 CPU.
 */
#define __isa_ge_and_ase(isa, ase)	((LOONGARCH_ISA_REV >= (isa)) && __ase(ase))
#define __isa_ge_and_opt(isa, opt)	((LOONGARCH_ISA_REV >= (isa)) && __opt(opt))

/*
 * Check if LOONGARCH_ISA_REV is >= isa *or* an option or ASE is detected during
 * boot (typically by cpu_probe()).
 *
 * These are for use with features that are optional up until a particular ISA
 * revision & then become required.
 */
#define __isa_ge_or_ase(isa, ase)	((LOONGARCH_ISA_REV >= (isa)) || __ase(ase))
#define __isa_ge_or_opt(isa, opt)	((LOONGARCH_ISA_REV >= (isa)) || __opt(opt))

/*
 * Check if LOONGARCH_ISA_REV is < isa *and* an option or ASE is detected during
 * boot (typically by cpu_probe()).
 *
 * These are for use with features that are optional up until a particular ISA
 * revision & are then removed - ie. no longer present in any CPU implementing
 * the given ISA revision.
 */
#define __isa_lt_and_ase(isa, ase)	((LOONGARCH_ISA_REV < (isa)) && __ase(ase))
#define __isa_lt_and_opt(isa, opt)	((LOONGARCH_ISA_REV < (isa)) && __opt(opt))

/*
 * SMP assumption: Options of CPU 0 are a superset of all processors.
 * This is true for all known LOONGARCH systems.
 */
#ifndef cpu_has_tlb
#define cpu_has_tlb		__opt(LOONGARCH_CPU_TLB)
#endif
#ifndef cpu_has_ftlb
#define cpu_has_ftlb		__opt(LOONGARCH_CPU_FTLB)
#endif
#ifndef cpu_has_tlbinv
#define cpu_has_tlbinv		__opt(LOONGARCH_CPU_TLBINV)
#endif
#ifndef cpu_has_segments
#define cpu_has_segments	__opt(LOONGARCH_CPU_SEGMENTS)
#endif
#ifndef cpu_has_eva
#define cpu_has_eva		__opt(LOONGARCH_CPU_EVA)
#endif
#ifndef cpu_has_ldpte
#define cpu_has_ldpte		__opt(LOONGARCH_CPU_LDPTE)
#endif
#ifndef cpu_has_rixiex
#define cpu_has_rixiex		__isa_ge_or_opt(6, LOONGARCH_CPU_RIXIEX)
#endif
#ifndef cpu_has_rw_llb
#define cpu_has_rw_llb		__isa_ge_or_opt(6, LOONGARCH_CPU_RW_LLB)
#endif

/*
 * For the moment we don't consider R6000 and R8000 so we can assume that
 * anything that doesn't support R4000-style exceptions and interrupts is
 * R3000-like.  Users should still treat these two macro definitions as
 * opaque.
 */
#ifndef cpu_has_3kex
#define cpu_has_3kex		(!cpu_has_4kex)
#endif
#ifndef cpu_has_4kex
#define cpu_has_4kex		__isa_ge_or_opt(1, LOONGARCH_CPU_4KEX)
#endif
#ifndef cpu_has_3k_cache
#define cpu_has_3k_cache	__isa_lt_and_opt(1, LOONGARCH_CPU_3K_CACHE)
#endif
#define cpu_has_6k_cache	0
#define cpu_has_8k_cache	0
#ifndef cpu_has_4k_cache
#define cpu_has_4k_cache	__isa_ge_or_opt(1, LOONGARCH_CPU_4K_CACHE)
#endif
#ifndef cpu_has_octeon_cache
#define cpu_has_octeon_cache	0
#endif
/* Don't override `cpu_has_fpu' to 1 or the "nofpu" option won't work.  */
#ifndef cpu_has_fpu
#define cpu_has_fpu		1
#define raw_cpu_has_fpu		1
#else
#define raw_cpu_has_fpu		cpu_has_fpu
#endif
#ifndef cpu_has_32fpr
#define cpu_has_32fpr		__isa_ge_or_opt(1, LOONGARCH_CPU_32FPR)
#endif
#ifndef cpu_has_counter
#define cpu_has_counter		__opt(LOONGARCH_CPU_COUNTER)
#endif
#ifndef cpu_has_watch
#define cpu_has_watch		__opt(LOONGARCH_CPU_WATCH)
#endif
#ifndef cpu_has_divec
#define cpu_has_divec		__isa_ge_or_opt(1, LOONGARCH_CPU_DIVEC)
#endif
#ifndef cpu_has_vce
#define cpu_has_vce		__opt(LOONGARCH_CPU_VCE)
#endif
#ifndef cpu_has_cache_cdex_p
#define cpu_has_cache_cdex_p	__opt(LOONGARCH_CPU_CACHE_CDEX_P)
#endif
#ifndef cpu_has_cache_cdex_s
#define cpu_has_cache_cdex_s	__opt(LOONGARCH_CPU_CACHE_CDEX_S)
#endif
#ifndef cpu_has_prefetch
#define cpu_has_prefetch	__isa_ge_or_opt(1, LOONGARCH_CPU_PREFETCH)
#endif
#ifndef cpu_has_mcheck
#define cpu_has_mcheck		__isa_ge_or_opt(1, LOONGARCH_CPU_MCHECK)
#endif
#ifndef cpu_has_ejtag
#define cpu_has_ejtag		__opt(LOONGARCH_CPU_EJTAG)
#endif
#ifndef cpu_has_llsc
#define cpu_has_llsc		__isa_ge_or_opt(1, LOONGARCH_CPU_LLSC)
#endif
#ifndef kernel_uses_llsc
#define kernel_uses_llsc	cpu_has_llsc
#endif
#ifndef cpu_has_guestid
#define cpu_has_guestid		__opt(LOONGARCH_CPU_GUESTID)
#endif
#ifndef cpu_has_cccp_guest
#define cpu_has_cccp_guest	(cpu_data[0].guest_cfg & (1 << 0))
#endif
#ifndef cpu_has_cccp_root
#define cpu_has_cccp_root	(cpu_data[0].guest_cfg & (1 << 1))
#endif
#ifndef cpu_has_cccp_nest
#define cpu_has_cccp_nest	(cpu_data[0].guest_cfg & (1 << 2))
#endif
#ifndef cpu_has_sitp
#define cpu_has_sitp	(cpu_data[0].guest_cfg & (1 << 6))
#endif
#ifndef cpu_has_titp
#define cpu_has_titp	(cpu_data[0].guest_cfg & (1 << 8))
#endif
#ifndef cpu_has_toep
#define cpu_has_toep	(cpu_data[0].guest_cfg & (1 << 10))
#endif
#ifndef cpu_has_topp
#define cpu_has_topp	(cpu_data[0].guest_cfg & (1 << 12))
#endif
#ifndef cpu_has_torup
#define cpu_has_torup	(cpu_data[0].guest_cfg & (1 << 14))
#endif
#ifndef cpu_has_gcip_all
#define cpu_has_gcip_all	(cpu_data[0].guest_cfg & (1 << 16))
#endif
#ifndef cpu_has_gcip_hit
#define cpu_has_gcip_hit	(cpu_data[0].guest_cfg & (1 << 17))
#endif
#ifndef cpu_has_gcip_secure
#define cpu_has_gcip_secure	(cpu_data[0].guest_cfg & (1 << 18))
#endif
#ifndef cpu_has_loongarch16
#define cpu_has_loongarch16		__isa_lt_and_ase(6, LOONGARCH_ASE_LOONGARCH16)
#endif
#ifndef cpu_has_loongarch16e2
#define cpu_has_loongarch16e2	__isa_lt_and_ase(6, LOONGARCH_ASE_LOONGARCH16E2)
#endif
#ifndef cpu_has_mdmx
#define cpu_has_mdmx		__isa_lt_and_ase(6, LOONGARCH_ASE_MDMX)
#endif
#ifndef cpu_has_loongarch3d
#define cpu_has_loongarch3d		__isa_lt_and_ase(6, LOONGARCH_ASE_LOONGARCH3D)
#endif

#ifndef cpu_has_lpa
#define cpu_has_lpa		__opt(LOONGARCH_CPU_LPA)
#endif
#ifndef cpu_has_mvh
#define cpu_has_mvh		__opt(LOONGARCH_CPU_MVH)
#endif
#ifndef cpu_has_xpa
#define cpu_has_xpa		(cpu_has_lpa && cpu_has_mvh)
#endif
#ifndef cpu_has_vtag_icache
#define cpu_has_vtag_icache	(cpu_data[0].icache.flags & LOONGARCH_CACHE_VTAG)
#endif
#ifndef cpu_has_dc_aliases
#define cpu_has_dc_aliases	(cpu_data[0].dcache.flags & LOONGARCH_CACHE_ALIASES)
#endif
#ifndef cpu_has_ic_fills_f_dc
#define cpu_has_ic_fills_f_dc	(cpu_data[0].icache.flags & LOONGARCH_CACHE_IC_F_DC)
#endif
#ifndef cpu_has_pindexed_dcache
#define cpu_has_pindexed_dcache	(cpu_data[0].dcache.flags & LOONGARCH_CACHE_PINDEX)
#endif
#ifndef cpu_has_local_ebase
#define cpu_has_local_ebase	1
#endif

/*
 * I-Cache snoops remote store.	 This only matters on SMP.  Some multiprocessors
 * such as the R10000 have I-Caches that snoop local stores; the embedded ones
 * don't.  For maintaining I-cache coherency this means we need to flush the
 * D-cache all the way back to whever the I-cache does refills from, so the
 * I-cache has a chance to see the new data at all.  Then we have to flush the
 * I-cache also.
 * Note we may have been rescheduled and may no longer be running on the CPU
 * that did the store so we can't optimize this into only doing the flush on
 * the local CPU.
 */
#ifndef cpu_icache_snoops_remote_store
#ifdef CONFIG_SMP
#define cpu_icache_snoops_remote_store	(cpu_data[0].icache.flags & LOONGARCH_IC_SNOOPS_REMOTE)
#else
#define cpu_icache_snoops_remote_store	1
#endif
#endif

/* __builtin_constant_p(cpu_has_loongarch_r) && cpu_has_loongarch_r */
#if !((defined(cpu_has_loongarch32r2) && cpu_has_loongarch32r2) || \
	  (defined(cpu_has_loongarch64r2) && cpu_has_loongarch64r2))
#define CPU_NO_EFFICIENT_FFS 1
#endif

#ifndef cpu_has_loongarch32r2
# define cpu_has_loongarch32r2	(cpu_data[0].isa_level & LOONGARCH_CPU_ISA_M32R2)
#endif
#ifndef cpu_has_loongarch64r2
# define cpu_has_loongarch64r2	(cpu_data[0].isa_level & LOONGARCH_CPU_ISA_M64R2)
#endif

/*
 * Shortcuts ...
 */
#define cpu_has_loongarch32	(cpu_has_loongarch32r2)
#define cpu_has_loongarch64	(cpu_has_loongarch64r2)
#define cpu_has_loongarch_r2 (cpu_has_loongarch32r2 | cpu_has_loongarch64r2)
#define cpu_has_loongarch_r	(cpu_has_loongarch32r2 | cpu_has_loongarch64r2)

/*
 * LOONGARCH32, LOONGARCH64, VR5500, IDT32332, IDT32334 and maybe a few other
 * pre-LOONGARCH32/LOONGARCH64 processors have CLO, CLZ.	The IDT RC64574 is 64-bit and
 * has CLO and CLZ but not DCLO nor DCLZ.  For 64-bit kernels
 * cpu_has_clo_clz also indicates the availability of DCLO and DCLZ.
 */
#ifndef cpu_has_clo_clz
#define cpu_has_clo_clz	cpu_has_loongarch_r
#endif

/*
 * LOONGARCH32 R2, LOONGARCH64 R2, Loongson 3A and Octeon have WSBH.
 * LOONGARCH64 R2, Loongson 3A and Octeon have WSBH, DSBH and DSHD.
 * This indicates the availability of WSBH and in case of 64 bit CPUs also
 * DSBH and DSHD.
 */
#ifndef cpu_has_wsbh
#define cpu_has_wsbh		cpu_has_loongarch_r2
#endif

#ifndef cpu_has_vp
#define cpu_has_vp		__isa_ge_and_opt(6, LOONGARCH_CPU_VP)
#endif

#ifdef CONFIG_32BIT
# ifndef cpu_has_nofpuex
# define cpu_has_nofpuex	__isa_lt_and_opt(1, LOONGARCH_CPU_NOFPUEX)
# endif
# ifndef cpu_has_64bits
# define cpu_has_64bits		(cpu_data[0].isa_level & LOONGARCH_CPU_ISA_64BIT)
# endif
# ifndef cpu_has_64bit_zero_reg
# define cpu_has_64bit_zero_reg	(cpu_data[0].isa_level & LOONGARCH_CPU_ISA_64BIT)
# endif
# ifndef cpu_has_64bit_gp_regs
# define cpu_has_64bit_gp_regs		0
# endif
# ifndef cpu_has_64bit_addresses
# define cpu_has_64bit_addresses	0
# endif
# ifndef cpu_vmbits
# define cpu_vmbits 31
# endif
#endif

#ifdef CONFIG_64BIT
# ifndef cpu_has_nofpuex
# define cpu_has_nofpuex		0
# endif
# ifndef cpu_has_64bits
# define cpu_has_64bits			1
# endif
# ifndef cpu_has_64bit_zero_reg
# define cpu_has_64bit_zero_reg		1
# endif
# ifndef cpu_has_64bit_gp_regs
# define cpu_has_64bit_gp_regs		1
# endif
# ifndef cpu_has_64bit_addresses
# define cpu_has_64bit_addresses	1
# endif
# ifndef cpu_vmbits
# define cpu_vmbits cpu_data[0].vmbits
# define __NEED_VMBITS_PROBE
# endif
#endif

#if defined(CONFIG_CPU_LOONGARCHR2_IRQ_VI) && !defined(cpu_has_vint)
# define cpu_has_vint		__opt(LOONGARCH_CPU_VINT)
#elif !defined(cpu_has_vint)
# define cpu_has_vint			0
#endif

#if defined(CONFIG_CPU_LOONGARCHR2_IRQ_EI) && !defined(cpu_has_veic)
# define cpu_has_veic		__opt(LOONGARCH_CPU_VEIC)
#elif !defined(cpu_has_veic)
# define cpu_has_veic			0
#endif

#ifndef cpu_has_inclusive_pcaches
#define cpu_has_inclusive_pcaches	__opt(LOONGARCH_CPU_INCLUSIVE_CACHES)
#endif

#ifndef cpu_dcache_line_size
#define cpu_dcache_line_size()	cpu_data[0].dcache.linesz
#endif
#ifndef cpu_icache_line_size
#define cpu_icache_line_size()	cpu_data[0].icache.linesz
#endif
#ifndef cpu_scache_line_size
#define cpu_scache_line_size()	cpu_data[0].scache.linesz
#endif
#ifndef cpu_tcache_line_size
#define cpu_tcache_line_size()	cpu_data[0].tcache.linesz
#endif

#ifndef cpu_hwrena_impl_bits
#define cpu_hwrena_impl_bits		0
#endif

#ifndef cpu_has_perf_cntr_intr_bit
#define cpu_has_perf_cntr_intr_bit	__opt(LOONGARCH_CPU_PCI)
#endif

#ifndef cpu_has_vz
#define cpu_has_vz		__ase(LOONGARCH_ASE_VZ)
#endif

#if defined(CONFIG_CPU_HAS_MSA) && !defined(cpu_has_msa)
# define cpu_has_msa		1
#elif !defined(cpu_has_msa)
# define cpu_has_msa		0
#endif

#if defined(CONFIG_CPU_HAS_LASX) && !defined(cpu_has_lasx)
# define cpu_has_lasx		1
#elif !defined(cpu_has_lasx)
# define cpu_has_lasx		0
#endif

#if defined(CONFIG_CPU_HAS_LBT) && !defined(cpu_has_lbt)
# define cpu_has_lbt		1
#elif !defined(cpu_has_lbt)
# define cpu_has_lbt		0
#endif

#ifndef cpu_has_ufr
# define cpu_has_ufr		__opt(LOONGARCH_CPU_UFR)
#endif

#ifndef cpu_has_fre
# define cpu_has_fre		__opt(LOONGARCH_CPU_FRE)
#endif

#ifndef cpu_has_cdmm
# define cpu_has_cdmm		__opt(LOONGARCH_CPU_CDMM)
#endif

#ifndef cpu_has_small_pages
# define cpu_has_small_pages	__opt(LOONGARCH_CPU_SP)
#endif

#ifndef cpu_has_nan_legacy
#define cpu_has_nan_legacy	__isa_lt_and_opt(6, LOONGARCH_CPU_NAN_LEGACY)
#endif
#ifndef cpu_has_nan_2008
#define cpu_has_nan_2008	__isa_ge_or_opt(6, LOONGARCH_CPU_NAN_2008)
#endif

#ifndef cpu_has_ebase_wg
# define cpu_has_ebase_wg	__opt(LOONGARCH_CPU_EBASE_WG)
#endif

#ifndef cpu_has_badinstr
# define cpu_has_badinstr	__isa_ge_or_opt(6, LOONGARCH_CPU_BADINSTR)
#endif

#ifndef cpu_has_badinstrp
# define cpu_has_badinstrp	__isa_ge_or_opt(6, LOONGARCH_CPU_BADINSTRP)
#endif

#ifndef cpu_has_nestfeatures
#define cpu_has_nestfeatures       (cpu_data[0].options & LOONGARCH_CPU_NF)
#endif

#ifndef cpu_has_contextconfig
# define cpu_has_contextconfig	__opt(LOONGARCH_CPU_CTXTC)
#endif

#ifndef cpu_has_perf
# define cpu_has_perf		__opt(LOONGARCH_CPU_PERF)
#endif

#ifdef CONFIG_SMP
/*
 * Some systems share FTLB RAMs between threads within a core (siblings in
 * kernel parlance). This means that FTLB entries may become invalid at almost
 * any point when an entry is evicted due to a sibling thread writing an entry
 * to the shared FTLB RAM.
 *
 * This is only relevant to SMP systems, and the only systems that exhibit this
 * property implement LOONGARCHr6 or higher so we constrain support for this to
 * kernels that will run on such systems.
 */
# ifndef cpu_has_shared_ftlb_ram
#  define cpu_has_shared_ftlb_ram \
	__isa_ge_and_opt(6, LOONGARCH_CPU_SHARED_FTLB_RAM)
# endif

/*
 * Some systems take this a step further & share FTLB entries between siblings.
 * This is implemented as TLB writes happening as usual, but if an entry
 * written by a sibling exists in the shared FTLB for a translation which would
 * otherwise cause a TLB refill exception then the CPU will use the entry
 * written by its sibling rather than triggering a refill & writing a matching
 * TLB entry for itself.
 *
 * This is naturally only valid if a TLB entry is known to be suitable for use
 * on all siblings in a CPU, and so it only takes effect when MMIDs are in use
 * rather than ASIDs or when a TLB entry is marked global.
 */
# ifndef cpu_has_shared_ftlb_entries
#  define cpu_has_shared_ftlb_entries \
	__isa_ge_and_opt(6, LOONGARCH_CPU_SHARED_FTLB_ENTRIES)
# endif
#endif /* SMP */

#ifndef cpu_has_shared_ftlb_ram
# define cpu_has_shared_ftlb_ram 0
#endif
#ifndef cpu_has_shared_ftlb_entries
# define cpu_has_shared_ftlb_entries 0
#endif

#if defined(CONFIG_CPU_LOONGSON3)
#ifndef cpu_has_lamo
#define cpu_has_lamo    (cpu_data[0].lses & LOONGARCH_HAS_LSE_LAMO)
#endif
#endif
/*
 * Guest capabilities
 */
#ifndef cpu_guest_has_conf1
#define cpu_guest_has_conf1	(cpu_data[0].guest.conf & (1 << 1))
#endif
#ifndef cpu_guest_has_conf2
#define cpu_guest_has_conf2	(cpu_data[0].guest.conf & (1 << 2))
#endif
#ifndef cpu_guest_has_conf3
#define cpu_guest_has_conf3	(cpu_data[0].guest.conf & (1 << 3))
#endif
#ifndef cpu_guest_has_fpu
#define cpu_guest_has_fpu	(cpu_data[0].guest.options & LOONGARCH_CPU_FPU)
#endif
#ifndef cpu_guest_has_watch
#define cpu_guest_has_watch	(cpu_data[0].guest.options & LOONGARCH_CPU_WATCH)
#endif
#ifndef cpu_guest_has_msa
#define cpu_guest_has_msa	(cpu_data[0].guest.ases & LOONGARCH_ASE_MSA)
#endif
#ifndef cpu_guest_has_kscr
#define cpu_guest_has_kscr(n)	(cpu_data[0].guest.kscratch_mask & (1u << (n)))
#endif
#ifndef cpu_guest_has_rw_llb
#define cpu_guest_has_rw_llb	((cpu_data[0].guest.options & LOONGARCH_CPU_RW_LLB))
#endif
#ifndef cpu_guest_has_perf
#define cpu_guest_has_perf	(cpu_data[0].guest.options & LOONGARCH_CPU_PERF)
#endif

/*
 * Guest dynamic capabilities
 */
#ifndef cpu_guest_has_dyn_fpu
#define cpu_guest_has_dyn_fpu	(cpu_data[0].guest.options_dyn & LOONGARCH_CPU_FPU)
#endif
#ifndef cpu_guest_has_dyn_perf
#define cpu_guest_has_dyn_perf	(cpu_data[0].guest.options_dyn & LOONGARCH_CPU_PERF)
#endif
#ifndef cpu_guest_has_dyn_msa
#define cpu_guest_has_dyn_msa	(cpu_data[0].guest.ases_dyn & LOONGARCH_ASE_MSA)
#endif

#endif /* __ASM_CPU_FEATURES_H */
