/* SPDX-License-Identifier: GPL-2.0 */
/*
 * cpu.h: Values of the PRId register used to match up
 *	  various LOONGARCH cpu types.
 *
 * Copyright (C) 1996 David S. Miller (davem@davemloft.net)
 * Copyright (C) 2004, 2013  Maciej W. Rozycki
 * Copyright (C) 2020 Loongson Technology Corporation Limited
 */
#ifndef _ASM_CPU_H
#define _ASM_CPU_H

/*
   As of the LOONGARCH32 and LOONGARCH64 specs from MTI, the PRId register (CP0
   register 15, select 0) is defined in this (backwards compatible) way:

  +----------------+----------------+----------------+----------------+
  | Company Options| Company ID	    | Processor ID   | Revision	      |
  +----------------+----------------+----------------+----------------+
   31		 24 23		  16 15		    8 7

   I don't have docs for all the previous processors, but my impression is
   that bits 16-23 have been 0 for all LOONGARCH processors before the LOONGARCH32/64
   spec.
*/

#define PRID_OPT_MASK		0xff000000

/*
 * Assigned Company values for bits 23:16 of the PRId register.
 */

#define PRID_COMP_MASK		0xff0000

#define PRID_COMP_LEGACY	0x000000
#define PRID_COMP_LOONGARCH		0x010000
#define PRID_COMP_BROADCOM	0x020000
#define PRID_COMP_ALCHEMY	0x030000
#define PRID_COMP_SIBYTE	0x040000
#define PRID_COMP_SANDCRAFT	0x050000
#define PRID_COMP_NXP		0x060000
#define PRID_COMP_TOSHIBA	0x070000
#define PRID_COMP_LSI		0x080000
#define PRID_COMP_LEXRA		0x0b0000
#define PRID_COMP_NETLOGIC	0x0c0000
#define PRID_COMP_CAVIUM	0x0d0000
#define PRID_COMP_LOONGSON	0x140000
#define PRID_COMP_INGENIC_D0	0xd00000	/* JZ4740, JZ4750 */
#define PRID_COMP_INGENIC_D1	0xd10000	/* JZ4770, JZ4775 */
#define PRID_COMP_INGENIC_E1	0xe10000	/* JZ4780 */

/*
 * Assigned Processor ID (implementation) values for bits 15:8 of the PRId
 * register.  In order to detect a certain CPU type exactly eventually
 * additional registers may need to be examined.
 */

#define PRID_IMP_MASK		0xff00

/*
 * These are valid when 23:16 == PRID_COMP_LEGACY
 */

#define PRID_IMP_R2000		0x0100
#define PRID_IMP_AU1_REV1	0x0100
#define PRID_IMP_AU1_REV2	0x0200
#define PRID_IMP_R6000		0x0300		/* Same as R3000A  */
#define PRID_IMP_R4000		0x0400
#define PRID_IMP_R6000A		0x0600
#define PRID_IMP_R10000		0x0900
#define PRID_IMP_R4300		0x0b00
#define PRID_IMP_VR41XX		0x0c00
#define PRID_IMP_R12000		0x0e00
#define PRID_IMP_R14000		0x0f00		/* R14K && R16K */
#define PRID_IMP_R8000		0x1000
#define PRID_IMP_PR4450		0x1200
#define PRID_IMP_R4600		0x2000
#define PRID_IMP_R4700		0x2100
#define PRID_IMP_R4640		0x2200
#define PRID_IMP_R4650		0x2200		/* Same as R4640 */
#define PRID_IMP_R5000		0x2300
#define PRID_IMP_TX49		0x2d00
#define PRID_IMP_SONIC		0x2400
#define PRID_IMP_MAGIC		0x2500
#define PRID_IMP_RM7000		0x2700
#define PRID_IMP_NEVADA		0x2800		/* RM5260 ??? */
#define PRID_IMP_RM9000		0x3400
#define PRID_IMP_LOONGSON_32	0x4200  /* Loongson-1 */
#define PRID_IMP_R5432		0x5400
#define PRID_IMP_R5500		0x5500
#define PRID_IMP_LOONGSON_64	0x6300  /* Loongson-2/3 */

#define PRID_IMP_UNKNOWN	0xff00

/*
 * Particular Revision values for bits 7:0 of the PRId register.
 */

#define PRID_REV_MASK		0x00ff

/*
 * Definitions for 7:0 on legacy processors
 */

#define PRID_REV_LOONGSON3A_R1		0x0005
#define PRID_REV_LOONGSON3B_R1		0x0006
#define PRID_REV_LOONGSON3B_R2		0x0007
#define PRID_REV_LOONGSON3A_R2_0	0x0008
#define PRID_REV_LOONGSON3A_R3_0	0x0009
#define PRID_REV_LOONGSON3A_R2_1	0x000c
#define PRID_REV_LOONGSON3A_R3_1	0x000d
#define PRID_REV_LOONGSON3A_R4_0	0xc000

/*
 * Older processors used to encode processor version and revision in two
 * 4-bit bitfields, the 4K seems to simply count up and even newer MTI cores
 * have switched to use the 8-bits as 3:3:2 bitfield with the last field as
 * the patch number.  *ARGH*
 */
#define PRID_REV_ENCODE_44(ver, rev)					\
	((ver) << 4 | (rev))
#define PRID_REV_ENCODE_332(ver, rev, patch)				\
	((ver) << 5 | (rev) << 2 | (patch))

/*
 * FPU implementation/revision register (CP1 control register 0).
 *
 * +---------------------------------+----------------+----------------+
 * | 0				     | Implementation | Revision       |
 * +---------------------------------+----------------+----------------+
 *  31				   16 15	     8 7	      0
 */

#define FPIR_IMP_MASK		0xff00

#define FPIR_IMP_NONE		0x0000

#if !defined(__ASSEMBLY__)

enum cpu_type_enum {
	CPU_UNKNOWN,

	/*
	 * LOONGARCH64 class processors
	 */
	CPU_LOONGSON3, CPU_LOONGSON3_COMP,

	CPU_LAST
};

#endif /* !__ASSEMBLY */

/*
 * ISA Level encodings
 *
 */
#define LOONGARCH_CPU_ISA_II		0x00000001
#define LOONGARCH_CPU_ISA_III	0x00000002
#define LOONGARCH_CPU_ISA_IV		0x00000004
#define LOONGARCH_CPU_ISA_V		0x00000008
#define LOONGARCH_CPU_ISA_M32R1	0x00000010
#define LOONGARCH_CPU_ISA_M32R2	0x00000020
#define LOONGARCH_CPU_ISA_M64R1	0x00000040
#define LOONGARCH_CPU_ISA_M64R2	0x00000080
#define LOONGARCH_CPU_ISA_M32R6	0x00000100
#define LOONGARCH_CPU_ISA_M64R6	0x00000200

#define LOONGARCH_CPU_ISA_32BIT (LOONGARCH_CPU_ISA_II | LOONGARCH_CPU_ISA_M32R1 | \
	LOONGARCH_CPU_ISA_M32R2 | LOONGARCH_CPU_ISA_M32R6)
#define LOONGARCH_CPU_ISA_64BIT (LOONGARCH_CPU_ISA_III | LOONGARCH_CPU_ISA_IV | \
	LOONGARCH_CPU_ISA_V | LOONGARCH_CPU_ISA_M64R1 | LOONGARCH_CPU_ISA_M64R2 | \
	LOONGARCH_CPU_ISA_M64R6)

/*
 * Private version of BIT_ULL() to escape include file recursion hell.
 * We soon will have to switch to another mechanism that will work with
 * more than 64 bits anyway.
 */
#define MBIT_ULL(bit)		(1ULL << (bit))

/*
 * CPU Option encodings
 */
#define LOONGARCH_CPU_TLB		MBIT_ULL( 0)	/* CPU has TLB */
#define LOONGARCH_CPU_4KEX		MBIT_ULL( 1)	/* "R4K" exception model */
#define LOONGARCH_CPU_4K_CACHE	MBIT_ULL( 3)	/* R4000-style caches */
#define LOONGARCH_CPU_FPU		MBIT_ULL( 5)	/* CPU has FPU */
#define LOONGARCH_CPU_32FPR		MBIT_ULL( 6)	/* 32 dbl. prec. FP registers */
#define LOONGARCH_CPU_COUNTER	MBIT_ULL( 7)	/* Cycle count/compare */
#define LOONGARCH_CPU_WATCH		MBIT_ULL( 8)	/* watchpoint registers */
#define LOONGARCH_CPU_DIVEC		MBIT_ULL( 9)	/* dedicated interrupt vector */
#define LOONGARCH_CPU_VCE		MBIT_ULL(10)	/* virt. coherence conflict possible */
#define LOONGARCH_CPU_CACHE_CDEX_P	MBIT_ULL(11)	/* Create_Dirty_Exclusive CACHE op */
#define LOONGARCH_CPU_CACHE_CDEX_S	MBIT_ULL(12)	/* ... same for seconary cache ... */
#define LOONGARCH_CPU_MCHECK		MBIT_ULL(13)	/* Machine check exception */
#define LOONGARCH_CPU_EJTAG		MBIT_ULL(14)	/* EJTAG exception */
#define LOONGARCH_CPU_NOFPUEX	MBIT_ULL(15)	/* no FPU exception */
#define LOONGARCH_CPU_LLSC		MBIT_ULL(16)	/* CPU has ll/sc instructions */
#define LOONGARCH_CPU_INCLUSIVE_CACHES	MBIT_ULL(17)	/* P-cache subset enforced */
#define LOONGARCH_CPU_PREFETCH	MBIT_ULL(18)	/* CPU has usable prefetch */
#define LOONGARCH_CPU_VINT		MBIT_ULL(19)	/* CPU supports LOONGARCHR2 vectored interrupts */
#define LOONGARCH_CPU_VEIC		MBIT_ULL(20)	/* CPU supports LOONGARCHR2 external interrupt controller mode */
#define LOONGARCH_CPU_ULRI		MBIT_ULL(21)	/* CPU has ULRI feature */
#define LOONGARCH_CPU_PCI		MBIT_ULL(22)	/* CPU has Perf Ctr Int indicator */
#define LOONGARCH_CPU_RIXI		MBIT_ULL(23)	/* CPU has TLB Read/eXec Inhibit */
#define LOONGARCH_CPU_MICROLOONGARCH	MBIT_ULL(24)	/* CPU has microLOONGARCH capability */
#define LOONGARCH_CPU_TLBINV		MBIT_ULL(25)	/* CPU supports TLBINV/F */
#define LOONGARCH_CPU_SEGMENTS	MBIT_ULL(26)	/* CPU supports Segmentation Control registers */
#define LOONGARCH_CPU_EVA		MBIT_ULL(27)	/* CPU supports Enhanced Virtual Addressing */
#define LOONGARCH_CPU_HTW		MBIT_ULL(28)	/* CPU support Hardware Page Table Walker */
#define LOONGARCH_CPU_RIXIEX		MBIT_ULL(29)	/* CPU has unique exception codes for {Read, Execute}-Inhibit exceptions */
#define LOONGARCH_CPU_MAAR		MBIT_ULL(30)	/* MAAR(I) registers are present */
#define LOONGARCH_CPU_FRE		MBIT_ULL(31)	/* FRE & UFE bits implemented */
#define LOONGARCH_CPU_RW_LLB		MBIT_ULL(32)	/* LLADDR/LLB writes are allowed */
#define LOONGARCH_CPU_LPA		MBIT_ULL(33)	/* CPU supports Large Physical Addressing */
#define LOONGARCH_CPU_CDMM		MBIT_ULL(34)	/* CPU has Common Device Memory Map */
#define LOONGARCH_CPU_BP_GHIST	MBIT_ULL(35)	/* R12K+ Branch Prediction Global History */
#define LOONGARCH_CPU_SP		MBIT_ULL(36)	/* Small (1KB) page support */
#define LOONGARCH_CPU_FTLB		MBIT_ULL(37)	/* CPU has Fixed-page-size TLB */
#define LOONGARCH_CPU_NAN_LEGACY	MBIT_ULL(38)	/* Legacy NaN implemented */
#define LOONGARCH_CPU_NAN_2008	MBIT_ULL(39)	/* 2008 NaN implemented */
#define LOONGARCH_CPU_VP		MBIT_ULL(40)	/* LOONGARCHr6 Virtual Processors (multi-threading) */
#define LOONGARCH_CPU_LDPTE		MBIT_ULL(41)	/* CPU has ldpte/lddir instructions */
#define LOONGARCH_CPU_MVH		MBIT_ULL(42)	/* CPU supports MFHC0/MTHC0 */
#define LOONGARCH_CPU_EBASE_WG	MBIT_ULL(43)	/* CPU has EBase.WG */
#define LOONGARCH_CPU_BADINSTR	MBIT_ULL(44)	/* CPU has BadInstr register */
#define LOONGARCH_CPU_BADINSTRP	MBIT_ULL(45)	/* CPU has BadInstrP register */
#define LOONGARCH_CPU_CTXTC		MBIT_ULL(46)	/* CPU has [X]ConfigContext registers */
#define LOONGARCH_CPU_PERF		MBIT_ULL(47)	/* CPU has LOONGARCH performance counters */
#define LOONGARCH_CPU_GUESTCTL0EXT	MBIT_ULL(48)	/* CPU has VZ GuestCtl0Ext register */
#define LOONGARCH_CPU_GUESTCTL1	MBIT_ULL(49)	/* CPU has VZ GuestCtl1 register */
#define LOONGARCH_CPU_GUESTCTL2	MBIT_ULL(50)	/* CPU has VZ GuestCtl2 register */
#define LOONGARCH_CPU_GUESTID	MBIT_ULL(51)	/* CPU uses VZ ASE GuestID feature */
#define LOONGARCH_CPU_DRG		MBIT_ULL(52)	/* CPU has VZ Direct Root to Guest (DRG) */
#define LOONGARCH_CPU_UFR		MBIT_ULL(53)	/* CPU supports User mode FR switching */
#define LOONGARCH_CPU_SHARED_FTLB_RAM \
				MBIT_ULL(54)	/* CPU shares FTLB RAM with another */
#define LOONGARCH_CPU_SHARED_FTLB_ENTRIES \
				MBIT_ULL(55)	/* CPU shares FTLB entries with another */
#define LOONGARCH_CPU_MT_PER_TC_PERF_COUNTERS \
				MBIT_ULL(56)	/* CPU has perf counters implemented per TC (LOONGARCHMT ASE) */
#define LOONGARCH_CPU_NF		MBIT_ULL(63)	/* CPU supports Nested Fault registers */

/*
 * CPU ASE encodings
 */
#define LOONGARCH_ASE_LOONGARCH16		0x00000001 /* code compression */
#define LOONGARCH_ASE_MDMX		0x00000002 /* LOONGARCH digital media extension */
#define LOONGARCH_ASE_LOONGARCH3D		0x00000004 /* LOONGARCH-3D */
#define LOONGARCH_ASE_SMARTLOONGARCH	0x00000008 /* SmartLOONGARCH */
#define LOONGARCH_ASE_DSP		0x00000010 /* Signal Processing ASE */
#define LOONGARCH_ASE_LOONGARCHMT		0x00000020 /* CPU supports LOONGARCH MT */
#define LOONGARCH_ASE_DSP2P		0x00000040 /* Signal Processing ASE Rev 2 */
#define LOONGARCH_ASE_VZ		0x00000080 /* Virtualization ASE */
#define LOONGARCH_ASE_MSA		0x00000100 /* LOONGARCH SIMD Architecture */
#define LOONGARCH_ASE_DSP3		0x00000200 /* Signal Processing ASE Rev 3*/
#define LOONGARCH_ASE_LOONGARCH16E2	0x00000400 /* LOONGARCH16e2 */

#if defined(CONFIG_CPU_LOONGSON3)
#define LOONGARCH_LSE_CPUCFG		MBIT_ULL(0) /* CPU supports cpucfg */
#define LOONGARCH_HAS_LSE_LAMO	MBIT_ULL(1) /* CPU supports lamo instructions */
#endif

#endif /* _ASM_CPU_H */
