/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995, 1996, 1997, 2000, 2001 by Ralf Baechle
 * Copyright (C) 2000 Silicon Graphics, Inc.
 * Modified for further R[236]000 support by Paul M. Antoine, 1996.
 * Kevin D. Kissell, kevink@mips.com and Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 2000, 07 MIPS Technologies, Inc.
 * Copyright (C) 2003, 2004  Maciej W. Rozycki
 * Copyright (C) 2020 Loongson Technology Co., Ltd.
 */
#ifndef _ASM_LOONGARCHREGS_H
#define _ASM_LOONGARCHREGS_H

#include <linux/linkage.h>
#include <linux/types.h>
#include <asm/isa-rev.h>
#include <loongarch_csr.h>

/*
 * The following macros are especially useful for __asm__
 * inline assembler.
 */
#ifndef __STR
#define __STR(x) #x
#endif
#ifndef STR
#define STR(x) __STR(x)
#endif

/*
 *  Configure language
 */
#ifdef __ASSEMBLY__
#define _ULCAST_
#define _U64CAST_
#else
#define _ULCAST_ (unsigned long)
#define _U64CAST_ (u64)
#endif

/* Generic EntryLo bit definitions */
#define ENTRYLO_V		(_ULCAST_(1) << 0)
#define ENTRYLO_D		(_ULCAST_(1) << 1)
#define ENTRYLO_C_SHIFT		4
#define ENTRYLO_C		(_ULCAST_(3) << ENTRYLO_C_SHIFT)
#define ENTRYLO_G		(_ULCAST_(1) << 6)

/*
 * LOONGARCHr6+ GlobalNumber register definitions
 */
#define LOONGARCH_GLOBALNUMBER_VP_SHF	0
#define LOONGARCH_GLOBALNUMBER_VP		(_ULCAST_(0xff) << LOONGARCH_GLOBALNUMBER_VP_SHF)
#define LOONGARCH_GLOBALNUMBER_CORE_SHF	8
#define LOONGARCH_GLOBALNUMBER_CORE		(_ULCAST_(0xff) << LOONGARCH_GLOBALNUMBER_CORE_SHF)
#define LOONGARCH_GLOBALNUMBER_CLUSTER_SHF	16
#define LOONGARCH_GLOBALNUMBER_CLUSTER	(_ULCAST_(0xf) << LOONGARCH_GLOBALNUMBER_CLUSTER_SHF)

/*
 * Values for PageMask register
 */
#define PM_4K		0x00000000
#define PM_8K		0x00002000
#define PM_16K		0x00006000
#define PM_32K		0x0000e000
#define PM_64K		0x0001e000
#define PM_128K		0x0003e000
#define PM_256K		0x0007e000
#define PM_512K		0x000fe000
#define PM_1M		0x001fe000
#define PM_2M		0x003fe000
#define PM_4M		0x007fe000
#define PM_8M		0x00ffe000
#define PM_16M		0x01ffe000
#define PM_32M		0x03ffe000
#define PM_64M		0x07ffe000
#define PM_256M		0x1fffe000
#define PM_1G		0x7fffe000

#define PS_4K		0x0000000c
#define PS_8K		0x0000000d
#define PS_16K		0x0000000e
#define PS_32K		0x0000000f
#define PS_64K		0x00000010
#define PS_128K		0x00000011
#define PS_256K		0x00000012
#define PS_512K		0x00000013
#define PS_1M		0x00000014
#define PS_2M		0x00000015
#define PS_4M		0x00000016
#define PS_8M		0x00000017
#define PS_16M		0x00000018
#define PS_32M		0x00000019
#define PS_64M		0x0000001a
#define PS_256M		0x0000001b
#define PS_1G		0x0000001c

#define PS_MASK		0x3f000000
#define PS_SHIFT	24

/*
 * Default page size for a given kernel configuration
 */
#ifdef CONFIG_PAGE_SIZE_4KB
#define PM_DEFAULT_MASK PM_4K
#elif defined(CONFIG_PAGE_SIZE_8KB)
#define PM_DEFAULT_MASK PM_8K
#elif defined(CONFIG_PAGE_SIZE_16KB)
#define PM_DEFAULT_MASK PM_16K
#elif defined(CONFIG_PAGE_SIZE_32KB)
#define PM_DEFAULT_MASK PM_32K
#elif defined(CONFIG_PAGE_SIZE_64KB)
#define PM_DEFAULT_MASK PM_64K
#else
#error Bad page size configuration!
#endif

#ifdef CONFIG_PAGE_SIZE_4KB
#define PS_DEFAULT_SIZE PS_4K
#elif defined(CONFIG_PAGE_SIZE_8KB)
#define PS_DEFAULT_SIZE PS_8K
#elif defined(CONFIG_PAGE_SIZE_16KB)
#define PS_DEFAULT_SIZE PS_16K
#elif defined(CONFIG_PAGE_SIZE_32KB)
#define PS_DEFAULT_SIZE PS_32K
#elif defined(CONFIG_PAGE_SIZE_64KB)
#define PS_DEFAULT_SIZE PS_64K
#else
#error Bad page size configuration!
#endif

/*
 * Default huge tlb size for a given kernel configuration
 */
#ifdef CONFIG_PAGE_SIZE_4KB
#define PM_HUGE_MASK   PM_1M
#elif defined(CONFIG_PAGE_SIZE_8KB)
#define PM_HUGE_MASK   PM_4M
#elif defined(CONFIG_PAGE_SIZE_16KB)
#define PM_HUGE_MASK   PM_16M
#elif defined(CONFIG_PAGE_SIZE_32KB)
#define PM_HUGE_MASK   PM_64M
#elif defined(CONFIG_PAGE_SIZE_64KB)
#define PM_HUGE_MASK   PM_256M
#else
#error Bad page size configuration for hugetlbfs!
#endif

#ifdef CONFIG_PAGE_SIZE_4KB
#define PS_HUGE_SIZE   PS_1M
#elif defined(CONFIG_PAGE_SIZE_8KB)
#define PS_HUGE_SIZE   PS_4M
#elif defined(CONFIG_PAGE_SIZE_16KB)
#define PS_HUGE_SIZE   PS_16M
#elif defined(CONFIG_PAGE_SIZE_32KB)
#define PS_HUGE_SIZE   PS_64M
#elif defined(CONFIG_PAGE_SIZE_64KB)
#define PS_HUGE_SIZE   PS_256M
#else
#error Bad page size configuration for hugetlbfs!
#endif

/*
 * Values used for computation of new tlb entries
 */
#define PL_4K		12
#define PL_16K		14
#define PL_64K		16
#define PL_256K		18
#define PL_1M		20
#define PL_4M		22
#define PL_16M		24
#define PL_64M		26
#define PL_256M		28

/* LOONGARCH32/64 EntryHI bit definitions */
#define LOONGARCH_ENTRYHI_EHINV	(_ULCAST_(1) << 10)
#define LOONGARCH_ENTRYHI_ASIDX	(_ULCAST_(0x3) << 8)
#define LOONGARCH_ENTRYHI_ASID	(_ULCAST_(0xff) << 0)

/*
 * ExStatus.ExcCode
 */
#define EXCCODE_RSV			0	/* Reserved */
#define EXCCODE_TLBL		1	/* TLB miss on a load */
#define EXCCODE_TLBS		2	/* TLB miss on a store */
#define EXCCODE_TLBI		3	/* TLB miss on a ifetch */
#define EXCCODE_MOD			4	/* TLB modified fault */
#define EXCCODE_TLBRI		5	/* TLB Read-Inhibit exception */
#define EXCCODE_TLBXI		6	/* TLB Execution-Inhibit exception */
#define EXCCODE_TLBPE		7	/* TLB Privilege Error */
#define EXCCODE_ADE		8	/* TODO: ???? */
	#define EXSUBCODE_ADEF		0	/* Fetch Instruction */
	#define EXSUBCODE_ADEM		1	/* Access Memory*/
#define EXCCODE_UNALIGN		9	/* TODO: ???? */
#define EXCCODE_OOB		10  /* TODO: ???? out of bounds */
#define EXCCODE_SYS		11	/* System call */
#define EXCCODE_BP		12	/* Breakpoint */
#define EXCCODE_RI		13	/* Reserved inst */
#define EXCCODE_IPE		14	/* Inst. Privileged Error */
#define EXCCODE_FPDIS		15	/* TODO: ???? */
#define EXCCODE_LSXDIS		16  /* TODO: ???? */
#define EXCCODE_LASXDIS		17	/* TODO: ???? */
#define EXCCODE_FPE		18	/* Floating Point Exception */
	#define EXCSUBCODE_FPE		0	/* Floating Point Exception */
	#define EXCSUBCODE_VFPE		1	/* Vector Exception */
#define EXCCODE_WATCH		19	/* Watch address reference */
#define EXCCODE_BTDIS		20	/* TODO: ???? */
#define EXCCODE_BTE		21	/* Binary Trans. Exception */
#define EXCCODE_PSI		22	/* TODO: ???? */
#define EXCCODE_HYP		23	/* TODO: ???? */
#define EXCCODE_FC		24	/* TODO: ???? */
	#define EXCSUBCODE_GCSC		0	/* TODO: ???? */
	#define EXCSUBCODE_GCHC		1	/* TODO: ???? */
#define EXCCODE_SE		25	/* TODO: ???? */

#define EXCCODE_INT_START   64
#define EXCCODE_SIP0        64
#define EXCCODE_SIP1        65
#define EXCCODE_IP0         66
#define EXCCODE_IP1         67
#define EXCCODE_IP2         68
#define EXCCODE_IP3         69
#define EXCCODE_IP4         70
#define EXCCODE_IP5         71
#define EXCCODE_IP6         72
#define EXCCODE_IP7         73
#define EXCCODE_PC          74 /* Performance Counter */
#define EXCCODE_TIMER       75
#define EXCCODE_IPI         76
#define EXCCODE_NMI         77
#define EXCCODE_INT_END     78

/* Implementation specific trap codes used by LOONGARCH cores */
#define LOONGARCH_EXCCODE_TLBPAR	16	/* TLB parity error exception */

/* Implementation specific trap codes used by Loongson-64 cores */
#define LS64_EXCCODE_GSEX	16	/* LS64 secondary exception identified by GSCause */

/* Bits specific to the LOONGARCH32/64 PRA.	*/
#define LOONGARCH_CONF_VI		(_ULCAST_(1) <<  3)
#define LOONGARCH_CONF_MT		(_ULCAST_(7) <<	 7)
#define LOONGARCH_CONF_MT_TLB	(_ULCAST_(1) <<  7)
#define LOONGARCH_CONF_MT_FTLB	(_ULCAST_(4) <<  7)
#define LOONGARCH_CONF_AR		(_ULCAST_(7) << 10)
#define LOONGARCH_CONF_AT		(_ULCAST_(3) << 13)
#define LOONGARCH_CONF_LASXEN	(_ULCAST_(1) << 19)
#define LOONGARCH_CONF_M		(_ULCAST_(1) << 31)

/* Bits specific to the Loongson-64.  */
#define LS64_CONF_CAMEN		(_ULCAST_(1) << 16)
#define LS64_CONF_QMEMP		(_ULCAST_(1) << 17)
#define LS64_CONF_LASXEN	(_ULCAST_(1) << 19)
#define LS64_CONF_LBTEN		(_ULCAST_(1) << 20)

/*
 * Bits in the LOONGARCH32/64 PRA coprocessor 0 config registers 1 and above.
 */
#define LOONGARCH_CONF1_FP		(_ULCAST_(1) <<	 0)
#define LOONGARCH_CONF1_EP		(_ULCAST_(1) <<	 1)
#define LOONGARCH_CONF1_CA		(_ULCAST_(1) <<	 2)
#define LOONGARCH_CONF1_WR		(_ULCAST_(1) <<	 3)
#define LOONGARCH_CONF1_PC		(_ULCAST_(1) <<	 4)
#define LOONGARCH_CONF1_MD		(_ULCAST_(1) <<	 5)
#define LOONGARCH_CONF1_C2		(_ULCAST_(1) <<	 6)
#define LOONGARCH_CONF1_DA_SHF	7
#define LOONGARCH_CONF1_DA_SZ	3
#define LOONGARCH_CONF1_DA		(_ULCAST_(7) <<	 7)
#define LOONGARCH_CONF1_DL_SHF	10
#define LOONGARCH_CONF1_DL_SZ	3
#define LOONGARCH_CONF1_DL		(_ULCAST_(7) << 10)
#define LOONGARCH_CONF1_DS_SHF	13
#define LOONGARCH_CONF1_DS_SZ	3
#define LOONGARCH_CONF1_DS		(_ULCAST_(7) << 13)
#define LOONGARCH_CONF1_IA_SHF	16
#define LOONGARCH_CONF1_IA_SZ	3
#define LOONGARCH_CONF1_IA		(_ULCAST_(7) << 16)
#define LOONGARCH_CONF1_IL_SHF	19
#define LOONGARCH_CONF1_IL_SZ	3
#define LOONGARCH_CONF1_IL		(_ULCAST_(7) << 19)
#define LOONGARCH_CONF1_IS_SHF	22
#define LOONGARCH_CONF1_IS_SZ	3
#define LOONGARCH_CONF1_IS		(_ULCAST_(7) << 22)
#define LOONGARCH_CONF1_TLBS_SHIFT   (25)
#define LOONGARCH_CONF1_TLBS_SIZE    (6)
#define LOONGARCH_CONF1_TLBS         (_ULCAST_(63) << LOONGARCH_CONF1_TLBS_SHIFT)

#define LOONGARCH_CONF2_SA		(_ULCAST_(15)<<	 0)
#define LOONGARCH_CONF2_SL		(_ULCAST_(15)<<	 4)
#define LOONGARCH_CONF2_SS		(_ULCAST_(15)<<	 8)
#define LOONGARCH_CONF2_SU		(_ULCAST_(15)<< 12)
#define LOONGARCH_CONF2_TA		(_ULCAST_(15)<< 16)
#define LOONGARCH_CONF2_TL		(_ULCAST_(15)<< 20)
#define LOONGARCH_CONF2_TS		(_ULCAST_(15)<< 24)
#define LOONGARCH_CONF2_TU		(_ULCAST_(7) << 28)

#define LOONGARCH_CONF3_TL		(_ULCAST_(1) <<	 0)
#define LOONGARCH_CONF3_SM		(_ULCAST_(1) <<	 1)
#define LOONGARCH_CONF3_MT		(_ULCAST_(1) <<	 2)
#define LOONGARCH_CONF3_CDMM		(_ULCAST_(1) <<	 3)
#define LOONGARCH_CONF3_SP		(_ULCAST_(1) <<	 4)
#define LOONGARCH_CONF3_VINT		(_ULCAST_(1) <<	 5)
#define LOONGARCH_CONF3_VEIC		(_ULCAST_(1) <<	 6)
#define LOONGARCH_CONF3_LPA		(_ULCAST_(1) <<	 7)
#define LOONGARCH_CONF3_ITL		(_ULCAST_(1) <<	 8)
#define LOONGARCH_CONF3_CTXTC	(_ULCAST_(1) <<	 9)
#define LOONGARCH_CONF3_DSP		(_ULCAST_(1) << 10)
#define LOONGARCH_CONF3_DSP2P	(_ULCAST_(1) << 11)
#define LOONGARCH_CONF3_RXI		(_ULCAST_(1) << 12)
#define LOONGARCH_CONF3_ULRI		(_ULCAST_(1) << 13)
#define LOONGARCH_CONF3_ISA		(_ULCAST_(3) << 14)
#define LOONGARCH_CONF3_ISA_OE	(_ULCAST_(1) << 16)
#define LOONGARCH_CONF3_MCU		(_ULCAST_(1) << 17)
#define LOONGARCH_CONF3_MMAR		(_ULCAST_(7) << 18)
#define LOONGARCH_CONF3_IPLW		(_ULCAST_(3) << 21)
#define LOONGARCH_CONF3_VZ		(_ULCAST_(1) << 23)
#define LOONGARCH_CONF3_PW		(_ULCAST_(1) << 24)
#define LOONGARCH_CONF3_SC		(_ULCAST_(1) << 25)
#define LOONGARCH_CONF3_BI		(_ULCAST_(1) << 26)
#define LOONGARCH_CONF3_BP		(_ULCAST_(1) << 27)
#define LOONGARCH_CONF3_MSA		(_ULCAST_(1) << 28)
#define LOONGARCH_CONF3_CMGCR	(_ULCAST_(1) << 29)
#define LOONGARCH_CONF3_BPG		(_ULCAST_(1) << 30)

#define LOONGARCH_CONF4_MMUSIZEEXT_SHIFT	(0)
#define LOONGARCH_CONF4_MMUSIZEEXT	(_ULCAST_(255) << 0)
#define LOONGARCH_CONF4_FTLBSETS_SHIFT	(0)
#define LOONGARCH_CONF4_FTLBSETS	(_ULCAST_(15) << LOONGARCH_CONF4_FTLBSETS_SHIFT)
#define LOONGARCH_CONF4_FTLBWAYS_SHIFT	(4)
#define LOONGARCH_CONF4_FTLBWAYS	(_ULCAST_(15) << LOONGARCH_CONF4_FTLBWAYS_SHIFT)
#define LOONGARCH_CONF4_FTLBPAGESIZE_SHIFT	(8)
/* bits 10:8 in FTLB-only configurations */
#define LOONGARCH_CONF4_FTLBPAGESIZE (_ULCAST_(7) << LOONGARCH_CONF4_FTLBPAGESIZE_SHIFT)
/* bits 12:8 in VTLB-FTLB only configurations */
#define LOONGARCH_CONF4_VFTLBPAGESIZE (_ULCAST_(31) << LOONGARCH_CONF4_FTLBPAGESIZE_SHIFT)
#define LOONGARCH_CONF4_MMUEXTDEF	(_ULCAST_(3) << 14)
#define LOONGARCH_CONF4_MMUEXTDEF_MMUSIZEEXT (_ULCAST_(1) << 14)
#define LOONGARCH_CONF4_MMUEXTDEF_FTLBSIZEEXT	(_ULCAST_(2) << 14)
#define LOONGARCH_CONF4_MMUEXTDEF_VTLBSIZEEXT	(_ULCAST_(3) << 14)
#define LOONGARCH_CONF4_KSCREXIST_SHIFT	(16)
#define LOONGARCH_CONF4_KSCREXIST	(_ULCAST_(255) << LOONGARCH_CONF4_KSCREXIST_SHIFT)
#define LOONGARCH_CONF4_VTLBSIZEEXT_SHIFT	(24)
#define LOONGARCH_CONF4_VTLBSIZEEXT	(_ULCAST_(15) << LOONGARCH_CONF4_VTLBSIZEEXT_SHIFT)
#define LOONGARCH_CONF4_AE		(_ULCAST_(1) << 28)
#define LOONGARCH_CONF4_IE		(_ULCAST_(3) << 29)
#define LOONGARCH_CONF4_TLBINV	(_ULCAST_(2) << 29)

#define LOONGARCH_CONF5_NF		(_ULCAST_(1) << 0)
#define LOONGARCH_CONF5_UFR		(_ULCAST_(1) << 2)
#define LOONGARCH_CONF5_MRP		(_ULCAST_(1) << 3)
#define LOONGARCH_CONF5_LLB		(_ULCAST_(1) << 4)
#define LOONGARCH_CONF5_MVH		(_ULCAST_(1) << 5)
#define LOONGARCH_CONF5_VP		(_ULCAST_(1) << 7)
#define LOONGARCH_CONF5_SBRI		(_ULCAST_(1) << 6)
#define LOONGARCH_CONF5_FRE		(_ULCAST_(1) << 8)
#define LOONGARCH_CONF5_UFE		(_ULCAST_(1) << 9)
#define LOONGARCH_CONF5_CA2		(_ULCAST_(1) << 14)
#define LOONGARCH_CONF5_CRCP		(_ULCAST_(1) << 18)
#define LOONGARCH_CONF5_MSAEN	(_ULCAST_(1) << 27)
#define LOONGARCH_CONF5_EVA		(_ULCAST_(1) << 28)
#define LOONGARCH_CONF5_CV		(_ULCAST_(1) << 29)
#define LOONGARCH_CONF5_K		(_ULCAST_(1) << 30)

#define LOONGARCH_CONF6_DATAPREF	(_ULCAST_(1) << 0)
#define LOONGARCH_CONF6_INSTPREF	(_ULCAST_(1) << 1)
#define LOONGARCH_CONF6_STPREFCTL	(_ULCAST_(3) << 2)
#define LOONGARCH_CONF6_DISBTB	(_ULCAST_(1) << 5)
#define LOONGARCH_CONF6_INNTIMER	(_ULCAST_(1) << 6)
#define LOONGARCH_CONF6_EXTIMER	(_ULCAST_(1) << 7)
#define LOONGARCH_CONF6_SFBEN	(_ULCAST_(1) << 8)
#define LOONGARCH_CONF6_UMEMUALEN	(_ULCAST_(1) << 10)
#define LOONGARCH_CONF6_DISBLKLYEN	(_ULCAST_(1) << 11)
#define LOONGARCH_CONF6_PIXUEN	(_ULCAST_(1) << 12)
#define LOONGARCH_CONF6_SYND		(_ULCAST_(1) << 13)
#define LOONGARCH_CONF6_VCLRU	(_ULCAST_(1) << 14)
#define LOONGARCH_CONF6_LLEXCEN	(_ULCAST_(1) << 16)
#define LOONGARCH_CONF6_SCRAND	(_ULCAST_(1) << 17)
#define LOONGARCH_CONF6_PIXNUEN	(_ULCAST_(1) << 18)
#define LOONGARCH_CONF6_DISRDTIME	(_ULCAST_(1) << 19)
#define LOONGARCH_CONF6_SSEN		(_ULCAST_(1) << 20)
#define LOONGARCH_CONF6_KE		(_ULCAST_(1) << 23)
#define LOONGARCH_CONF6_KPOS		(_ULCAST_(0x3f) << 24)
#define LOONGARCH_CONF6_BPPASS	(_ULCAST_(1) << 31)
/* proAptiv FTLB on/off bit */
#define LOONGARCH_CONF6_FTLBEN	(_ULCAST_(1) << 15)
/* Loongson-3 FTLB on/off bit */
#define LOONGARCH_CONF6_FTLBDIS	(_ULCAST_(1) << 22)
#define LOONGARCH_CONF6_LASXMODE	(_ULCAST_(1) << 21)
/* FTLB probability bits */
#define LOONGARCH_CONF6_FTLBP_SHIFT	(16)

/* WatchLo* register definitions */
#define LOONGARCH_WATCHLO_IRW	(_ULCAST_(0x7) << 0)

/* WatchHi* register definitions */
#define LOONGARCH_WATCHHI_M		(_ULCAST_(1) << 31)
#define LOONGARCH_WATCHHI_G		(_ULCAST_(1) << 30)
#define LOONGARCH_WATCHHI_WM		(_ULCAST_(0x3) << 28)
#define LOONGARCH_WATCHHI_WM_R_RVA	(_ULCAST_(0) << 28)
#define LOONGARCH_WATCHHI_WM_R_GPA	(_ULCAST_(1) << 28)
#define LOONGARCH_WATCHHI_WM_G_GVA	(_ULCAST_(2) << 28)
#define LOONGARCH_WATCHHI_EAS	(_ULCAST_(0x3) << 24)
#define LOONGARCH_WATCHHI_ASID	(_ULCAST_(0xff) << 16)
#define LOONGARCH_WATCHHI_MASK	(_ULCAST_(0x1ff) << 3)
#define LOONGARCH_WATCHHI_I		(_ULCAST_(1) << 2)
#define LOONGARCH_WATCHHI_R		(_ULCAST_(1) << 1)
#define LOONGARCH_WATCHHI_W		(_ULCAST_(1) << 0)
#define LOONGARCH_WATCHHI_IRW	(_ULCAST_(0x7) << 0)

/*
 * FPU register names
 */
#define LOONGARCH_FCSR0	$r0
#define LOONGARCH_FCSR1	$r1
#define LOONGARCH_FCSR2	$r2
#define LOONGARCH_FCSR3	$r3

/*
 * Bits in the LOONGARCH32/64 coprocessor 1 (FPU) condition codes register.
 */
#define LOONGARCH_FCCR_CONDX_S	0
#define LOONGARCH_FCCR_CONDX		(_ULCAST_(255) << LOONGARCH_FCCR_CONDX_S)
#define LOONGARCH_FCCR_COND0_S	0
#define LOONGARCH_FCCR_COND0		(_ULCAST_(1) << LOONGARCH_FCCR_COND0_S)
#define LOONGARCH_FCCR_COND1_S	1
#define LOONGARCH_FCCR_COND1		(_ULCAST_(1) << LOONGARCH_FCCR_COND1_S)
#define LOONGARCH_FCCR_COND2_S	2
#define LOONGARCH_FCCR_COND2		(_ULCAST_(1) << LOONGARCH_FCCR_COND2_S)
#define LOONGARCH_FCCR_COND3_S	3
#define LOONGARCH_FCCR_COND3		(_ULCAST_(1) << LOONGARCH_FCCR_COND3_S)
#define LOONGARCH_FCCR_COND4_S	4
#define LOONGARCH_FCCR_COND4		(_ULCAST_(1) << LOONGARCH_FCCR_COND4_S)
#define LOONGARCH_FCCR_COND5_S	5
#define LOONGARCH_FCCR_COND5		(_ULCAST_(1) << LOONGARCH_FCCR_COND5_S)
#define LOONGARCH_FCCR_COND6_S	6
#define LOONGARCH_FCCR_COND6		(_ULCAST_(1) << LOONGARCH_FCCR_COND6_S)
#define LOONGARCH_FCCR_COND7_S	7
#define LOONGARCH_FCCR_COND7		(_ULCAST_(1) << LOONGARCH_FCCR_COND7_S)

/*
 * Bits in the LOONGARCH32/64 coprocessor 1 (FPU) enables register.
 */
#define LOONGARCH_FENR_FS_S		2
#define LOONGARCH_FENR_FS		(_ULCAST_(1) << LOONGARCH_FENR_FS_S)

/*
 * FPU Status Register Values
 */
#define FPU_CSR_COND_S	23					/* $fcc0 */
#define FPU_CSR_COND	(_ULCAST_(1) << FPU_CSR_COND_S)

#define FPU_CSR_FS_S	24		/* flush denormalised results to 0 */
#define FPU_CSR_FS	(_ULCAST_(1) << FPU_CSR_FS_S)

#define FPU_CSR_CONDX_S	25					/* $fcc[7:1] */
#define FPU_CSR_CONDX	(_ULCAST_(127) << FPU_CSR_CONDX_S)
#define FPU_CSR_COND1_S	25					/* $fcc1 */
#define FPU_CSR_COND1	(_ULCAST_(1) << FPU_CSR_COND1_S)
#define FPU_CSR_COND2_S	26					/* $fcc2 */
#define FPU_CSR_COND2	(_ULCAST_(1) << FPU_CSR_COND2_S)
#define FPU_CSR_COND3_S	27					/* $fcc3 */
#define FPU_CSR_COND3	(_ULCAST_(1) << FPU_CSR_COND3_S)
#define FPU_CSR_COND4_S	28					/* $fcc4 */
#define FPU_CSR_COND4	(_ULCAST_(1) << FPU_CSR_COND4_S)
#define FPU_CSR_COND5_S	29					/* $fcc5 */
#define FPU_CSR_COND5	(_ULCAST_(1) << FPU_CSR_COND5_S)
#define FPU_CSR_COND6_S	30					/* $fcc6 */
#define FPU_CSR_COND6	(_ULCAST_(1) << FPU_CSR_COND6_S)
#define FPU_CSR_COND7_S	31					/* $fcc7 */
#define FPU_CSR_COND7	(_ULCAST_(1) << FPU_CSR_COND7_S)

/*
 * Bits 22:20 of the FPU Status Register will be read as 0,
 * and should be written as zero.
 */
#define FPU_CSR_RSVD	(_ULCAST_(7) << 20)

#define FPU_CSR_ABS2008	(_ULCAST_(1) << 19)
#define FPU_CSR_NAN2008	(_ULCAST_(1) << 18)

/*
 * X the exception cause indicator
 * E the exception enable
 * S the sticky/flag bit
*/
#define FPU_CSR_ALL_X	0x1f000000
#define FPU_CSR_INV_X	0x10000000
#define FPU_CSR_DIV_X	0x08000000
#define FPU_CSR_OVF_X	0x04000000
#define FPU_CSR_UDF_X	0x02000000
#define FPU_CSR_INE_X	0x01000000

#define FPU_CSR_ALL_S	0x001f0000
#define FPU_CSR_INV_S	0x00100000
#define FPU_CSR_DIV_S	0x00080000
#define FPU_CSR_OVF_S	0x00040000
#define FPU_CSR_UDF_S	0x00020000
#define FPU_CSR_INE_S	0x00010000

#define FPU_CSR_ALL_E	0x0000001f
#define FPU_CSR_INV_E	0x00000010
#define FPU_CSR_DIV_E	0x00000008
#define FPU_CSR_OVF_E	0x00000004
#define FPU_CSR_UDF_E	0x00000002
#define FPU_CSR_INE_E	0x00000001

/* Bits 0 and 1 of FPU Status Register specify the rounding mode */
#define FPU_CSR_RM	0x00000300
#define FPU_CSR_RN	0x000	/* nearest */
#define FPU_CSR_RZ	0x100	/* towards zero */
#define FPU_CSR_RU	0x200	/* towards +Infinity */
#define FPU_CSR_RD	0x300	/* towards -Infinity */


#ifndef __ASSEMBLY__
#include <lcsrintrin.h>

#define get_isa16_mode(x)		0
#define msk_isa16_mode(x)		(x)
#define set_isa16_mode(x)		do { } while(0)

/*
 * Helper macros for generating raw instruction encodings in inline asm.
 */
#define _ASM_INSN_IF_LOONGARCH(_enc)			\
	".word (" #_enc ")\n\t"

#ifndef _ASM_INSN16_IF_MM
#define _ASM_INSN16_IF_MM(_enc)
#endif
#ifndef _ASM_INSN32_IF_MM
#define _ASM_INSN32_IF_MM(_enc)
#endif
#ifndef _ASM_INSN_IF_LOONGARCH
#define _ASM_INSN_IF_LOONGARCH(_enc)
#endif

/*
 *read/write csr register
 *
 */
#ifdef CONFIG_CPU_LOONGSON3
static inline unsigned int read_cfg(int reg)
{
	unsigned int __res;

	__asm__ __volatile__(
		"parse_r __res,%0\n\t"
		"parse_r reg,%1\n\t"
		".word (0x6c00 | (reg << 5) | (__res))\n\t"
		:"=r"(__res)
		:"r"(reg)
		:
		);
	return __res;
}
#endif

/*
 * parse_r var, r - Helper assembler macro for parsing register names.
 *
 * This converts the register name in $n form provided in \r to the
 * corresponding register number, which is assigned to the variable \var. It is
 * needed to allow explicit encoding of instructions in inline assembly where
 * registers are chosen by the compiler in $n form, allowing us to avoid using
 * fixed register numbers.
 *
 * It also allows newer instructions (not implemented by the assembler) to be
 * transparently implemented using assembler macros, instead of needing separate
 * cases depending on toolchain support.
 *
 * Simple usage example:
 * __asm__ __volatile__("parse_r __rt, %0\n\t"
 *			"# di    %0\n\t"
 *			".word   (0x41606000 | (__rt << 16))"
 *			: "=r" (status);
 */

/* Match an individual register number and assign to \var */
#define _IFC_REG(n)				\
	".ifc	\\r, $r" #n "\n\t"		\
	"\\var	= " #n "\n\t"			\
	".endif\n\t"

__asm__(".macro	parse_r var r\n\t"
	"\\var	= -1\n\t"
	_IFC_REG(0)  _IFC_REG(1)  _IFC_REG(2)  _IFC_REG(3)
	_IFC_REG(4)  _IFC_REG(5)  _IFC_REG(6)  _IFC_REG(7)
	_IFC_REG(8)  _IFC_REG(9)  _IFC_REG(10) _IFC_REG(11)
	_IFC_REG(12) _IFC_REG(13) _IFC_REG(14) _IFC_REG(15)
	_IFC_REG(16) _IFC_REG(17) _IFC_REG(18) _IFC_REG(19)
	_IFC_REG(20) _IFC_REG(21) _IFC_REG(22) _IFC_REG(23)
	_IFC_REG(24) _IFC_REG(25) _IFC_REG(26) _IFC_REG(27)
	_IFC_REG(28) _IFC_REG(29) _IFC_REG(30) _IFC_REG(31)
	".iflt	\\var\n\t"
	".error	\"Unable to parse register name \\r\"\n\t"
	".endif\n\t"
	".endm");

#undef _IFC_REG

/*
 * C macros for generating assembler macros for common instruction formats.
 *
 * The names of the operands can be chosen by the caller, and the encoding of
 * register operand \<Rn> is assigned to __<Rn> where it can be accessed from
 * the ENC encodings.
 */

/* Instructions with no operands */
#define _ASM_MACRO_0(OP, ENC)						\
	__asm__(".macro	" #OP "\n\t"					\
		ENC							\
		".endm")

/* Instructions with 2 register operands */
#define _ASM_MACRO_2R(OP, R1, R2, ENC)					\
	__asm__(".macro	" #OP " " #R1 ", " #R2 "\n\t"			\
		"parse_r __" #R1 ", \\" #R1 "\n\t"			\
		"parse_r __" #R2 ", \\" #R2 "\n\t"			\
		ENC							\
		".endm")

/* Instructions with 3 register operands */
#define _ASM_MACRO_3R(OP, R1, R2, R3, ENC)				\
	__asm__(".macro	" #OP " " #R1 ", " #R2 ", " #R3 "\n\t"		\
		"parse_r __" #R1 ", \\" #R1 "\n\t"			\
		"parse_r __" #R2 ", \\" #R2 "\n\t"			\
		"parse_r __" #R3 ", \\" #R3 "\n\t"			\
		ENC							\
		".endm")

/* Instructions with 2 register operands and 1 optional select operand */
#define _ASM_MACRO_2R_1S(OP, R1, R2, SEL3, ENC)				\
	__asm__(".macro	" #OP " " #R1 ", " #R2 ", " #SEL3 " = 0\n\t"	\
		"parse_r __" #R1 ", \\" #R1 "\n\t"			\
		"parse_r __" #R2 ", \\" #R2 "\n\t"			\
		ENC							\
		".endm")

#ifdef CONFIG_CPU_LOONGSON3
#define GSEX_LASXDIS	0x7
#define GSEX_CODE(x)  ((x & 0x7c) >> 2)
#endif

/*
 * TLB Invalidate Flush
 */
static inline void tlbinvf(void)
{
	__asm__ __volatile__( "tlbflush");
}

static inline void tlbinv(void)
{
	__asm__ __volatile__( "tlbinv");
}

_ASM_MACRO_2R_1S(mfhc0, rt, rs, sel,
	_ASM_INSN_IF_LOONGARCH(0x40400000 | __rt << 16 | __rs << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x000000f4 | __rt << 21 | __rs << 16 | \\sel << 11));
_ASM_MACRO_2R_1S(mthc0, rt, rd, sel,
	_ASM_INSN_IF_LOONGARCH(0x40c00000 | __rt << 16 | __rd << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x000002f4 | __rt << 21 | __rd << 16 | \\sel << 11));

/*
 * Macros to access the guest system control coprocessor
 */

#ifndef TOOLCHAIN_SUPPORTS_VIRT
_ASM_MACRO_2R_1S(mfgc0, rt, rs, sel,
	_ASM_INSN_IF_LOONGARCH(0x40600000 | __rt << 16 | __rs << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x000004fc | __rt << 21 | __rs << 16 | \\sel << 11));
_ASM_MACRO_2R_1S(dmfgc0, rt, rs, sel,
	_ASM_INSN_IF_LOONGARCH(0x40600100 | __rt << 16 | __rs << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x580004fc | __rt << 21 | __rs << 16 | \\sel << 11));
_ASM_MACRO_2R_1S(mtgc0, rt, rd, sel,
	_ASM_INSN_IF_LOONGARCH(0x40600200 | __rt << 16 | __rd << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x000006fc | __rt << 21 | __rd << 16 | \\sel << 11));
_ASM_MACRO_2R_1S(dmtgc0, rt, rd, sel,
	_ASM_INSN_IF_LOONGARCH(0x40600300 | __rt << 16 | __rd << 11 | \\sel)
	_ASM_INSN32_IF_MM(0x580006fc | __rt << 21 | __rd << 16 | \\sel << 11));
_ASM_MACRO_0(gtlbp,    _ASM_INSN_IF_LOONGARCH(0x6482801));
_ASM_MACRO_0(gtlbr,    _ASM_INSN_IF_LOONGARCH(0x6482c01));
_ASM_MACRO_0(gtlbwi,   _ASM_INSN_IF_LOONGARCH(0x6483001));
_ASM_MACRO_0(gtlbwr,   _ASM_INSN_IF_LOONGARCH(0x6483401));
_ASM_MACRO_0(gtlbflush, _ASM_INSN_IF_LOONGARCH(0x6482401));
#define _ASM_SET_VIRT ""
#else	/* !TOOLCHAIN_SUPPORTS_VIRT */
#define _ASM_SET_VIRT ".set\tvirt\n\t"
#endif

#define write_fcsr(dest, val) \
do {	\
	__asm__ __volatile__(	\
	"	movgr2fcsr	%0, "STR(dest)"	\n"	\
	: : "r" (val));	\
} while (0)

#define read_fcsr(source)	\
({	\
	unsigned int __res;	\
\
	__asm__ __volatile__(	\
	"	movfcsr2gr	%0, "STR(source)"	\n"	\
	: "=r" (__res));	\
	__res;	\
})

/*
 * TLB operations.
 *
 */
static inline void tlb_probe(void)
{
	__asm__ __volatile__("tlbp");
}

static inline void tlb_read(void)
{
	__asm__ __volatile__("tlbr");
}

static inline void tlb_write_indexed(void)
{
	__asm__ __volatile__("tlbwi");
}

static inline void tlb_write_random(void)
{
	__asm__ __volatile__("tlbwr");
}

/*
 * Guest TLB operations.
 *
 */
static inline void guest_tlb_probe(void)
{
	__asm__ __volatile__(
		_ASM_SET_VIRT
		".word 0x6482801\n\t");
}

static inline void guest_tlb_read(void)
{
	__asm__ __volatile__(
		_ASM_SET_VIRT
		".word 0x6482c01\n\t");
}

static inline void guest_tlb_write_indexed(void)
{
	__asm__ __volatile__(
		_ASM_SET_VIRT
		".word 0x6483001\n\t");
}

static inline void guest_tlb_write_random(void)
{
	__asm__ __volatile__(
		_ASM_SET_VIRT
		".word 0x6483401\n\t");
}

/*
 * Guest TLB Invalidate Flush
 */
static inline void guest_tlbinvf(void)
{
	__asm__ __volatile__(
		_ASM_SET_VIRT
		".word 0x6482401\n\t");
}

static inline unsigned int get_ebase_cpunum(void)
{
	return __lcsr_dcsrrd(LOONGARCH_CSR_CPUN);
}

#endif /* !__ASSEMBLY__ */

#endif /* _ASM_LOONGARCHREGS_H */
